unit Acrops32;

interface

{ A silly wrapper component to hide the real code in the BsASpl32 unit }

{ Revisions:
     1/24/96  - Corrected problem in Create/Free not initializing properly
}

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;


type
  TAcropSpell = class(TComponent)
  private
    { Private declarations }
    DictDataPtr         : pointer;
    FIsDictionaryOpen   : boolean;
    FDictionaryMain     : string;
    FDictionaryUser     : string;
    FMaxSuggestions     : byte;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    procedure Free;
    function OpenDictionary(Filename : STRING) : boolean;
    function OpenUserDictionary(Filename : STRING) : integer;
    procedure CloseDictionaries;
    function GoodWord(TheWord : STRING) : BOOLEAN;
    function AddWord(TheWord : STRING; DictID : integer) : BOOLEAN;
    function SuggestCloseMatch(TheWord : STRING) : TStringList;
    function SuggestPhoneme(TheWord : STRING) : TStringList;
    function GetUserDictionaryList(DictID : integer) : TStringList;
    function CloseUserDictionary(DictID : INTEGER) : BOOLEAN;
    function DeleteUserDictionary(DictID : integer) : BOOLEAN;
    procedure DeleteUserDictionaries;
    procedure CloseUserDictionaries;
    function BuildUserDictionary(Filename : STRING; WordList : TStringList) : INTEGER;
    function IsDictionaryOpen : boolean;
    procedure SetMaxSuggestions(Max : byte);
  published
    { Published declarations }
    property DictionaryName : string read FDictionaryMain write FDictionaryMain;
    property DictionaryUser : string read FDictionaryUser write FDictionaryUser;
    property MaxSuggestions : byte read FMaxSuggestions write SetMaxSuggestions default 10;
  end;

procedure Register;

implementation

uses BsASpl32;

procedure Register;
begin
  RegisterComponents('Samples', [TAcropSpell]);
end;


constructor TAcropSpell.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FIsDictionaryOpen   := FALSE;
  FDictionaryMain     := 'acrop.dct';
  FDictionaryUser     := 'custom.dct';
  FMaxSuggestions     := 10;
  BsASpl32.InitDictionaryData(DictDataPtr);
end;

procedure TAcropSpell.Free;
begin
  if FIsDictionaryOpen then
    CloseDictionaries;
  BsASpl32.ReleaseDictionaryData(DictDataPtr);
  inherited Free;
end;

procedure OpenDefaultDictionary(AOwner : TAcropSpell);
begin
  with AOwner do
    begin
      FIsDictionaryOpen := OpenDictionary(FDictionaryMain);
    end;
end;

procedure TAcropSpell.SetMaxSuggestions(Max : byte);
begin
  if Max > 30 then
    Max := 30;
  FMaxSuggestions := Max;
end;

function TAcropSpell.OpenDictionary(Filename : STRING) : boolean;
begin
  OpenDictionary := FALSE;
  if FIsDictionaryOpen then
    exit;
  if BsASpl32.OpenDictionary(DictDataPtr, Filename) then
    begin
      FIsDictionaryOpen := TRUE;
      FDictionaryMain   := Filename;
      OpenDictionary    := TRUE;
    end;
end;

function TAcropSpell.OpenUserDictionary(Filename : STRING) : integer;
begin
  OpenUserDictionary := BsASpl32.OpenUserDictionary(DictDataPtr, Filename);
end;

procedure TAcropSpell.CloseDictionaries;
begin
  if not FIsDictionaryOpen then
    exit;
  BsASpl32.CloseDictionaries(DictDataPtr);
  FIsDictionaryOpen := FALSE;
end;

function TAcropSpell.GoodWord(TheWord : STRING) : BOOLEAN;
begin
  if not FIsDictionaryOpen then
    OpenDefaultDictionary(Self);
  GoodWord := BsASpl32.GoodWord(DictDataPtr, TheWord);
end;

function TAcropSpell.AddWord(TheWord : STRING; DictID : integer) : BOOLEAN;
begin
  if not FIsDictionaryOpen then
    OpenDefaultDictionary(Self);
  AddWord := BsASpl32.AddWord(DictDataPtr, TheWord, DictID);
end;

function TAcropSpell.SuggestCloseMatch(TheWord : STRING) : TStringList;
begin
  if not FIsDictionaryOpen then
    OpenDefaultDictionary(Self);
  SuggestCloseMatch := BsASpl32.SuggestCloseMatch(DictDataPtr, TheWord, FMaxSuggestions);
end;

function TAcropSpell.SuggestPhoneme(TheWord : STRING) : TStringList;
begin
  if not FIsDictionaryOpen then
    OpenDefaultDictionary(Self);
  SuggestPhoneme := BsASpl32.SuggestPhoneme(DictDataPtr, TheWord, FMaxSuggestions);
end;

function TAcropSpell.GetUserDictionaryList(DictID : integer) : TStringList;
begin
  GetUserDictionaryList := BsASpl32.GetUserDictionaryList(DictDataPtr, DictID);
end;

function TAcropSpell.CloseUserDictionary(DictID : INTEGER) : BOOLEAN;
begin
  CloseUserDictionary := BsASpl32.CloseUserDictionary(DictDataPtr, DictID);
end;

function TAcropSpell.DeleteUserDictionary(DictID : integer) : BOOLEAN;
begin
  DeleteUserDictionary := BsASpl32.DeleteUserDictionary(DictDataPtr, DictID);
end;

procedure TAcropSpell.DeleteUserDictionaries;
begin
  BsASpl32.DeleteUserDictionaries(DictDataPtr);
end;

procedure TAcropSpell.CloseUserDictionaries;
begin
  BsASpl32.CloseUserDictionaries(DictDataPtr);
end;

function TAcropSpell.BuildUserDictionary(Filename : STRING; WordList : TStringList) : integer;
begin
  BuildUserDictionary := BsASpl32.BuildUserDictionary(DictDataPtr, Filename, WordList);
end;

function TAcropSpell.IsDictionaryOpen : boolean;
begin
  IsDictionaryOpen := FIsDictionaryOpen;
end;

end.
